(* $Id: rCS.ml,v 1.5 2004/06/22 09:30:18 ohl Exp $ *)
(* Copyright (C) 2000-2004 by Thorsten Ohl <ohl@physik.uni-wuerzburg.de> et al.
   O'Mega is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by 
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
   O'Mega is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)  

type raw = { revision : string; date : string; author : string; source : string }

type t =
    { name : string;
      description : string list;
      rcs_revision : string;
      rcs_date : string;
      rcs_author : string;
      rcs_source : string }

let name r = r.name
let description r = r.description
let revision r = r.rcs_revision
let date r = r.rcs_date
let author r = r.rcs_author
let source r = r.rcs_source

let strip_dollars s =
  let l = String.length s in
  match s.[0], s.[l-1] with
  | '$', '$' -> String.sub s 1 (l-3)
  | '$', _ | _, '$' -> invalid_arg ("RCS.strip_dollars unpaired: " ^ s)
  | _ -> s

let strip_keyword k s =
  let lk = String.length k
  and ls = String.length s in
  if ls <= succ lk then
    invalid_arg ("RCS.strip_keyword: expected " ^ k ^ " got " ^ s)
  else if String.sub s 0 lk <> k || s.[lk] <> ':' then
    invalid_arg ("RCS.strip_keyword: expected " ^ k ^ " got " ^ s)
  else
    String.sub s (lk + 2) (ls - lk - 2)

let parse1 k s =
  strip_keyword k (strip_dollars s)

let parse name description r =
  { name = name;
    description = description;
    rcs_revision = parse1 "Revision" r.revision;
    rcs_date = parse1 "Date" r.date;
    rcs_author = parse1 "Author" r.author;
    rcs_source = parse1 "Source" r.source }

let rename rcs name description =
  { rcs with name = name; description = description }

let summary rcs =
  [ name rcs ^ ":"] @
  List.map (fun s -> "  " ^ s) (description rcs) @
  [ "  file: " ^ source rcs;
    "  revision: " ^ revision rcs ^ " checked in by " ^
    author rcs ^ " at " ^ date rcs ]

(*i
 *  Local Variables:
 *  mode:caml
 *  indent-tabs-mode:nil
 *  page-delimiter:"^(\\* .*\n"
 *  End:
i*)
